from flask import Blueprint, render_template, request
from flask_wtf import FlaskForm
from wtforms import RadioField
import sys
sys.path.append('~/eval')
from evalweb.web_checker import WebChecker, Language
from evalweb.examples import examples, convert_examples


bp = Blueprint('eval', __name__, url_prefix='/')

tasks = {'DFA': 'DFA',
         'TOT': 'Totální DFA',
         'MIN': 'Minimální DFA',
         'EFA': 'NFA (s ε-kroky)',
         'NFA': 'NFA bez ε-kroků',
         'GRA': 'Regulární gramatika',
         'REG': 'Regulární výraz'}

convs = {'DFA': 'DFA',
         'TOT': 'Totální DFA',
         'MIN': 'Minimální DFA',
         'EFA': 'NFA (s ε-kroky)',
         'NFA': 'NFA bez ε-kroků',
         'GRA': 'Regulární gramatika'}

types = {'DFA': 'DFA',
         'EFA': 'NFA (s ε-kroky)',
         'GRA': 'Regulární gramatika',
         'REG': 'Regulární výraz'}


class TypeForm(FlaskForm):
    make = RadioField('Typ', choices=list(types.items()), default='DFA', coerce=str)


class TaskForm(FlaskForm):
    make = RadioField('Task', choices=list(tasks.items()), default='DFA', coerce=str)

class ConvertForm(FlaskForm):
    make = RadioField('Convert', choices=list(convs.items()), default='DFA', coerce=str)


@bp.route('/compare', methods=('GET', 'POST'))
def compare():
    student_form = TaskForm(prefix='student_form')
    teacher_form = TypeForm(prefix='teacher_form')
    student_area = ""
    teacher_area = ""

    if request.method == 'POST' and 'submit_button' in request.form:
        teacher_type = teacher_form.make.data
        teacher_string = request.form['teacher_string']
        student_type = student_form.make.data
        student_string = request.form['student_string']
        student_area = student_string
        teacher_area = teacher_string

        checker = WebChecker(student_string=student_string, task=student_type)
        result = checker.compare(teacher_string=teacher_string, teacher_type=teacher_type)
        if not isinstance(result, bool):
            return render_template('parsing_error.html', error=result)

        extra_word_ce, missing_word_ce, inf = None, None, None
        if not result and checker.eq is not None:
            extra_word_ce = checker.eq.left_counterexample
            missing_word_ce = checker.eq.right_counterexample
            inf = checker.eq.inf

        return render_template('result_compare.html', compare=True, ok=result,
                               inf=inf, task_solved=checker.task_solved, alphabets=checker.alphabets,
                               extra_word_ce=extra_word_ce, missing_word_ce=missing_word_ce,
                               is_task=checker.is_task, img_src=checker.img_src,
                               teacher=checker.teacher, student=checker.student)

    if request.method == 'POST' and 'example_button' in request.form:
        teacher_type = teacher_form.make.data
        student_type = student_form.make.data
        if (teacher_type, student_type) in examples:
            teacher_area, student_area = examples[(teacher_type, student_type)]

    return render_template('compare.html', student_form=student_form, teacher_form=teacher_form,
                           student_area=student_area, teacher_area=teacher_area)


@bp.route('/convert', methods=('GET', 'POST'))
def convert():
    student_form = TypeForm(prefix='student_form')
    task_form = ConvertForm(prefix='task_form')
    student_area = ""

    if request.method == 'POST' and 'submit_button' in request.form:
        student_string = request.form['student_string']
        student_type = student_form.make.data
        task = task_form.make.data
        student_area = student_string

        checker = WebChecker(student_string=student_string, task=task)
        output = checker.convert(student_type=student_type)

        student = Language(string=student_string, task=student_type)
        is_task = student.gen_is_task(task=task)

        return render_template('result_convert.html', compare=False, student_string=student_string,
                               student_type=types[student_type], task=tasks[task], output=output, is_task=is_task)

    if request.method == 'POST' and 'example_button' in request.form:
        student_type = student_form.make.data
        if student_type in convert_examples:
            student_area = convert_examples[student_type]

    return render_template('convert.html', student_form=student_form, task_form=task_form, student_area=student_area)


@bp.route('/userref')
def userref():
    return render_template('userref.html')


@bp.route('/about')
def about():
    return render_template('about.html')